from io import StringIO
from unittest.mock import patch

from django.contrib.auth import get_user_model
from django.core.management import call_command
from django.test import TestCase, override_settings

from django_scrubber import scrubbers
from django_scrubber.management.commands.scrub_data import _get_model_scrubbers, _parse_scrubber_class_from_string

User = get_user_model()


class TestScrubData(TestCase):
    def setUp(self):
        self.user = User.objects.create(first_name="test_first_name")

    def test_scrub_data(self):
        with self.settings(DEBUG=True, SCRUBBER_GLOBAL_SCRUBBERS={"first_name": scrubbers.Faker("first_name")}):
            call_command("scrub_data", stdout=StringIO())
        self.user.refresh_from_db()

        self.assertNotEqual(self.user.first_name, "test_first_name")

    def test_scrub_data_callable_scrubber(self):
        with self.settings(DEBUG=True, SCRUBBER_GLOBAL_SCRUBBERS={"first_name": scrubbers.Hash}):
            call_command("scrub_data", stdout=StringIO())
        self.user.refresh_from_db()

        self.assertNotEqual(self.user.first_name, "test_first_name")

        # make sure it's a md5 hash
        self.assertRegex(self.user.first_name, "[a-f0-9]{32}")

    def test_scrub_data_debug_is_false(self):
        err = StringIO()

        with self.settings(DEBUG=False):
            call_command("scrub_data", stdout=StringIO(), stderr=err)
        output = err.getvalue()
        self.user.refresh_from_db()

        self.assertIn("This command should only be run with DEBUG=True, to avoid running on live systems", output)
        self.assertEqual(self.user.first_name, "test_first_name")

    @override_settings(SCRUBBER_STRICT_MODE=True)
    def test_scrub_data_strict_mode_enabled_scrubbing_blocked(self):
        err = StringIO()

        with self.settings(DEBUG=True):
            call_command("scrub_data", stdout=StringIO(), stderr=err)
        output = err.getvalue()
        self.user.refresh_from_db()

        self.assertIn(
            'When "SCRUBBER_STRICT_MODE" is enabled, '
            "you have to define a scrubbing policy for every text-based field.",
            output,
        )
        self.assertEqual(self.user.first_name, "test_first_name")

    def test_hash_simple_global_scrubber(self):
        with self.settings(DEBUG=True, SCRUBBER_GLOBAL_SCRUBBERS={"first_name": scrubbers.Hash}):
            call_command("scrub_data", stdout=StringIO())
        self.user.refresh_from_db()

        self.assertNotEqual(self.user.first_name, "test_first_name")

    def test_hash_simple_class_scrubber(self):
        class Scrubbers:
            first_name = scrubbers.Hash

        with self.settings(DEBUG=True), patch.object(User, "Scrubbers", Scrubbers, create=True):
            call_command("scrub_data", stdout=StringIO())
        self.user.refresh_from_db()

        self.assertNotEqual(self.user.first_name, "test_first_name")

    def test_scrub_invalid_field(self):
        class Scrubbers:
            this_does_not_exist_382784 = scrubbers.Null

        with (
            self.settings(DEBUG=True),
            patch.object(User, "Scrubbers", Scrubbers, create=True),
            self.assertWarnsRegex(
                Warning,
                "Scrubber defined for User.this_does_not_exist_382784 but field does not exist",
            ),
        ):
            call_command("scrub_data", stdout=StringIO())

    @override_settings(SCRUBBER_MAPPING={"auth.User": "example.scrubbers.UserScrubbers"})
    def test_get_model_scrubbers_mapper_from_settings_used(self):
        with (
            patch(
                "django_scrubber.management.commands.scrub_data._parse_scrubber_class_from_string",
                return_value={},
            ) as mocked_method,
            patch("django_scrubber.management.commands.scrub_data._get_fields", return_value=[]),
        ):
            test_scrubbers = _get_model_scrubbers(User)
        mocked_method.assert_called_once()
        self.assertEqual(test_scrubbers, {})

    def test_parse_scrubber_class_from_string_regular(self):
        class_type = _parse_scrubber_class_from_string("django_scrubber.tests.test_models.TestDjangoScrubber")
        self.assertIsInstance(class_type, type)

    def test_parse_scrubber_class_from_string_wrong_path(self):
        with self.assertRaises(ImportError):
            _parse_scrubber_class_from_string("not.valid.path")

    def test_parse_scrubber_class_from_string_path_no_separator(self):
        with self.assertRaises(ImportError):
            _parse_scrubber_class_from_string("broken_path")



from django.test import TestCase, override_settings

from django_scrubber import defaults, settings_with_fallback


class TestScrubbers(TestCase):
    def test_default(self):
        self.assertEqual(
            defaults["SCRUBBER_RANDOM_SEED"],
            settings_with_fallback("SCRUBBER_RANDOM_SEED"),
        )

    @override_settings(SCRUBBER_RANDOM_SEED=9001)
    def test_override(self):
        self.assertEqual(
            9001,
            settings_with_fallback("SCRUBBER_RANDOM_SEED"),
        )



import sys

from django.core.management.base import BaseCommand

from django_scrubber import settings_with_fallback
from django_scrubber.services.validator import ScrubberValidatorService


class Command(BaseCommand):
    def handle(self, *args, **options):
        service = ScrubberValidatorService()
        non_scrubbed_field_list = service.process()

        found_models = 0
        found_fields = 0

        if len(non_scrubbed_field_list):
            for model_path, affected_field_list in non_scrubbed_field_list.items():
                self.stdout.write(f"Model {model_path!r}:")

                found_models += 1
                for _field in affected_field_list:
                    self.stdout.write(f"- {_field}")
                    found_fields += 1

            self.stdout.write("")

            if found_models > 0:
                self.stdout.write(f"{found_models} model(s) having {found_fields} unscrubbed field(s) detected.")

                # strict mode should fail with a non-zero exit code
                if settings_with_fallback("SCRUBBER_STRICT_MODE"):
                    sys.exit(1)

                self.stdout.write("However strict mode is deactivated and scrubbing is not enforced.")
                sys.exit(0)

        self.stdout.write("No unscrubbed fields detected. Yeah!")



import re
from typing import Union

from django.apps import apps

from django_scrubber import settings_with_fallback


class ScrubberValidatorService:
    """
    Service to validate if all text-based fields are being scrubbed within your project and dependencies.
    """

    @staticmethod
    def check_pattern(pattern: Union[str, re.Pattern], value):
        if isinstance(pattern, str):
            return pattern == value
        if isinstance(pattern, re.Pattern):
            return pattern.fullmatch(value)
        raise ValueError("Invalid pattern type")

    def process(self) -> dict:
        from django_scrubber.management.commands.scrub_data import _get_model_scrubbers

        scrubber_required_field_types = settings_with_fallback("SCRUBBER_REQUIRED_FIELD_TYPES")
        model_whitelist = settings_with_fallback("SCRUBBER_REQUIRED_FIELD_MODEL_WHITELIST")

        # Get a list of all registered models in your Django application
        model_list = apps.get_models()

        # Create a dictionary to store the fields of each model
        non_scrubbed_field_list = {}

        # Iterate over each model in the list
        for model in model_list:
            # Check if model is whitelisted
            if any(self.check_pattern(pattern, model._meta.label) for pattern in model_whitelist):
                continue

            fields_need_scrubbing = []
            # Get the model's name and fields
            fields = model._meta.get_fields()

            # Gather list of all fields of the given model that require scrubbing
            for field in fields:
                if type(field) in scrubber_required_field_types:
                    fields_need_scrubbing.append(field.name)

            # Get scrubber class
            scrubber_class = _get_model_scrubbers(model)

            # If we did find a scrubber class...
            if scrubber_class:
                # We check for every scrubbing requiring field, if it's set to be scrubbed
                for scrubbed_field in scrubber_class:
                    if scrubbed_field.name in fields_need_scrubbing:
                        fields_need_scrubbing.remove(scrubbed_field.name)

            # Store per model all non-scrubbed but scrubbing requiring fields
            if len(fields_need_scrubbing) > 0:
                non_scrubbed_field_list[model._meta.label] = fields_need_scrubbing

        return non_scrubbed_field_list



from datetime import timedelta
from io import StringIO

import django
from django.contrib.sessions.models import Session
from django.core.management import call_command
from django.db import connection
from django.test import TestCase, override_settings
from django.utils import timezone

from django_scrubber import scrubbers
from django_scrubber.models import FakeData
from example.models import DataFactory, DataToBeScrubbed


class TestScrubbers(TestCase):
    def test_empty_scrubber(self):
        data = DataFactory.create(first_name="Foo")
        with self.settings(DEBUG=True, SCRUBBER_GLOBAL_SCRUBBERS={"first_name": scrubbers.Empty}):
            call_command("scrub_data", stdout=StringIO())
        data.refresh_from_db()

        self.assertEqual(data.first_name, "")

    def test_null_scrubber(self):
        data = DataFactory.create(date_past=timezone.now().date())
        with self.settings(DEBUG=True, SCRUBBER_GLOBAL_SCRUBBERS={"date_past": scrubbers.Null}):
            call_command("scrub_data", stdout=StringIO())
        data.refresh_from_db()

        self.assertEqual(data.date_past, None)

    def test_hash_scrubber_max_length(self):
        data = DataFactory.create(first_name="Foo")
        with self.settings(DEBUG=True, SCRUBBER_GLOBAL_SCRUBBERS={"first_name": scrubbers.Hash}):
            call_command("scrub_data", stdout=StringIO())
        data.refresh_from_db()

        self.assertNotEqual(data.first_name, "Foo")
        self.assertEqual(
            len(data.first_name),
            DataToBeScrubbed._meta.get_field("first_name").max_length,
            "len({}) != {}".format(data.first_name, DataToBeScrubbed._meta.get_field("first_name").max_length),
        )

    def test_hash_scrubber_textfield(self):
        data = DataFactory.create(description="Foo")
        with self.settings(DEBUG=True, SCRUBBER_GLOBAL_SCRUBBERS={"description": scrubbers.Hash}):
            call_command("scrub_data", stdout=StringIO())
        data.refresh_from_db()

        self.assertNotEqual(data.description, "Foo")

    def test_lorem_scrubber(self):
        data = DataFactory.create(description="Foo")
        with self.settings(DEBUG=True, SCRUBBER_GLOBAL_SCRUBBERS={"description": scrubbers.Lorem}):
            call_command("scrub_data", stdout=StringIO())
        data.refresh_from_db()

        self.assertNotEqual(data.description, "Foo")
        self.assertEqual(data.description[:11], "Lorem ipsum")

    def test_faker_scrubber_charfield(self):
        data = DataFactory.create(last_name="Foo")
        with self.settings(DEBUG=True, SCRUBBER_GLOBAL_SCRUBBERS={"last_name": scrubbers.Faker("last_name")}):
            call_command("scrub_data", stdout=StringIO())
        data.refresh_from_db()

        self.assertNotEqual(data.last_name, "Foo")
        self.assertNotEqual(data.last_name, "")

    def test_faker_scrubber_with_provider_arguments(self):
        """
        Use this as an example for Faker scrubbers with parameters passed along
        """
        data = DataFactory.create(ean8="8")
        with self.settings(DEBUG=True, SCRUBBER_GLOBAL_SCRUBBERS={"ean8": scrubbers.Faker("ean", length=8)}):
            call_command("scrub_data", stdout=StringIO())
        data.refresh_from_db()

        # The EAN Faker will by default emit ean13, so this would fail if the parameter was ignored
        self.assertEqual(8, len(data.ean8))

        # Add a new scrubber for ean13
        with self.settings(DEBUG=True, SCRUBBER_GLOBAL_SCRUBBERS={"ean8": scrubbers.Faker("ean", length=13)}):
            call_command("scrub_data", stdout=StringIO())
        data.refresh_from_db()

        # make sure it doesn't reuse the ean with length=8 scrubber
        self.assertEqual(13, len(data.ean8))

    def test_faker_scrubber_datefield(self):
        """
        Use this as an example for Scrubber's capability of optimistically Casting to the current field's type
        There is a bug with django < 2.1 and sqlite, that's why we don't run the test there.
        """
        if django.VERSION >= (2, 1) or connection.vendor != "sqlite":
            today = timezone.now().date()

            data = DataFactory.create(date_past=today)
            with self.settings(
                DEBUG=True,
                SCRUBBER_GLOBAL_SCRUBBERS={
                    "date_past": scrubbers.Faker(
                        "past_date",
                        start_date="-30d",
                        tzinfo=timezone.get_current_timezone(),
                    ),
                },
            ):
                call_command("scrub_data", stdout=StringIO())
            data.refresh_from_db()

            self.assertGreater(today, data.date_past)
            self.assertLess(today - timedelta(days=31), data.date_past)

    def test_faker_scrubber_run_twice(self):
        """
        Use this as an example of what happens when you want to run the same Faker scrubbers twice
        """
        data = DataFactory.create(company="Foo")
        with self.settings(DEBUG=True, SCRUBBER_GLOBAL_SCRUBBERS={"company": scrubbers.Faker("company")}):
            call_command("scrub_data", stdout=StringIO())
            call_command("scrub_data", stdout=StringIO())
        data.refresh_from_db()

        self.assertNotEqual(data.company, "Foo")
        self.assertNotEqual(data.company, "")

    @override_settings(DEBUG=True)
    def test_faker_scrubber_run_clear_session_by_default(self):
        """
        Ensures that the session table will be emptied by default
        """
        # Create session object
        Session.objects.create(session_key="foo", session_data="Lorem ipsum", expire_date=timezone.now())

        # Sanity check
        self.assertTrue(Session.objects.all().exists())

        # Call command
        call_command("scrub_data", stdout=StringIO())

        # Assertion that session table is empty now
        self.assertFalse(Session.objects.all().exists())

    @override_settings(DEBUG=True)
    def test_faker_scrubber_run_disable_session_clearing(self):
        """
        Ensures that the session table will be emptied by default
        """
        # Create session object
        Session.objects.create(session_key="foo", session_data="Lorem ipsum", expire_date=timezone.now())

        # Sanity check
        self.assertTrue(Session.objects.all().exists())

        # Call command
        call_command("scrub_data", keep_sessions=True, stdout=StringIO())

        # Assertion that session table is empty now
        self.assertTrue(Session.objects.all().exists())

    @override_settings(DEBUG=True)
    def test_faker_scrubber_run_clear_faker_data_not_by_default(self):
        """
        Ensure that scrubbing does not delete stored faker data unless explicitly requested.
        """
        # Create faker data object
        FakeData.objects.create(provider="company", content="Foo", provider_offset=1)

        # Sanity check
        self.assertTrue(FakeData.objects.filter(provider="company", content="Foo").exists())

        # Call command
        call_command("scrub_data", stdout=StringIO())

        # Assertion that faker data still exists
        self.assertTrue(FakeData.objects.filter(provider="company", content="Foo").exists())

    @override_settings(DEBUG=True)
    def test_faker_scrubber_run_clear_faker_data_works(self):
        """
        Ensures that fake data is cleared when requested
        """
        # Create faker data object
        FakeData.objects.create(provider="company", content="Foo", provider_offset=1)

        # Sanity check
        self.assertTrue(FakeData.objects.filter(provider="company", content="Foo").exists())

        # Call command
        call_command("scrub_data", remove_fake_data=True, stdout=StringIO())

        # Assertion that the faker data was removed
        self.assertFalse(FakeData.objects.filter(provider="company", content="Foo").exists())



import importlib
import warnings

from django.apps import apps
from django.conf import settings
from django.contrib.sessions.models import Session
from django.core.exceptions import FieldDoesNotExist
from django.core.management.base import BaseCommand, CommandError
from django.db.models import F
from django.db.utils import DataError, IntegrityError

from django_scrubber import settings_with_fallback
from django_scrubber.models import FakeData
from django_scrubber.scrubbers import Keep
from django_scrubber.services.validator import ScrubberValidatorService


class Command(BaseCommand):
    help = "Replace database data according to model-specific or global scrubbing rules."
    leave_locale_alone = True

    def add_arguments(self, parser):
        parser.add_argument(
            "--model",
            type=str,
            required=False,
            help="Scrub only a single model (format <app_label>.<model_name>)",
        )
        parser.add_argument(
            "--keep-sessions",
            action="store_true",
            required=False,
            help="Will NOT truncate all (by definition critical) session data",
        )
        parser.add_argument(
            "--remove-fake-data",
            action="store_true",
            required=False,
            help="Will truncate the database table storing preprocessed data for the Faker library. "
            "If you want to do multiple iterations of scrubbing, it will save you time to keep "
            "them. If not, you will add a huge bunch of data to your dump size.",
        )

    def handle(self, *args, **kwargs):
        if not settings.DEBUG:
            # avoid logger, otherwise we might silently fail if we're on live and logging is being sent somewhere else
            self.stderr.write("This command should only be run with DEBUG=True, to avoid running on live systems")
            return False

        # Check STRICT mode
        if settings_with_fallback("SCRUBBER_STRICT_MODE"):
            service = ScrubberValidatorService()
            non_scrubbed_field_list = service.process()
            if len(non_scrubbed_field_list) > 0:
                self.stderr.write(
                    'When "SCRUBBER_STRICT_MODE" is enabled, you have to define a scrubbing policy '
                    "for every text-based field.",
                )
                return False

        global_scrubbers = settings_with_fallback("SCRUBBER_GLOBAL_SCRUBBERS")

        # run only for selected model
        if kwargs.get("model") is not None:
            app_label, model_name = kwargs.get("model").rsplit(".", 1)
            try:
                models = [apps.get_model(app_label=app_label, model_name=model_name)]
            except LookupError as e:
                raise CommandError("--model should be defined as <app_label>.<model_name>") from e

        # run for all models of all apps
        else:
            models = apps.get_models()

        scrubber_apps_list = settings_with_fallback("SCRUBBER_APPS_LIST")
        for model_class in models:
            self._scrub_model(model_class, scrubber_apps_list, global_scrubbers)

        # Truncate session data
        if not kwargs.get("keep_sessions", False):
            Session.objects.all().delete()

        # Truncate Faker data
        if kwargs.get("remove_fake_data", False):
            FakeData.objects.all().delete()
            return None
        return None

    def _scrub_model(self, model_class, scrubber_apps_list, global_scrubbers):
        if model_class._meta.proxy:
            return
        if settings_with_fallback("SCRUBBER_SKIP_UNMANAGED") and not model_class._meta.managed:
            return
        if scrubber_apps_list and model_class._meta.app_config.name not in scrubber_apps_list:
            return

        scrubbers = {}
        for field in model_class._meta.fields:
            if field.name in global_scrubbers:
                scrubbers[field] = global_scrubbers[field.name]
            elif type(field) in global_scrubbers:
                scrubbers[field] = global_scrubbers[type(field)]

        scrubbers.update(_get_model_scrubbers(model_class))

        # Filter out all fields marked as "to be kept"
        scrubbers_without_kept_fields = {}
        for field, scrubbing_method in scrubbers.items():
            if scrubbing_method != Keep:
                scrubbers_without_kept_fields[field] = scrubbing_method
        scrubbers = scrubbers_without_kept_fields

        if not scrubbers:
            return

        realized_scrubbers = _filter_out_disabled(_call_callables(scrubbers))

        self.stdout.write(f"Scrubbing {model_class._meta.label} with {realized_scrubbers}")

        try:
            model_class.objects.annotate(
                mod_pk=F("pk") % settings_with_fallback("SCRUBBER_ENTRIES_PER_PROVIDER"),
            ).update(**realized_scrubbers)
        except IntegrityError as e:
            raise CommandError(
                f"Integrity error while scrubbing {model_class} ({e}); maybe increase SCRUBBER_ENTRIES_PER_PROVIDER?",
            ) from e
        except DataError as e:
            raise CommandError(f"DataError while scrubbing {model_class} ({e})") from e


def _call_callables(d):
    """
    Helper to realize lazy scrubbers, like Faker, or global field-type scrubbers
    """
    return {k.name: (v(k) if callable(v) else v) for k, v in d.items()}


def _parse_scrubber_class_from_string(path: str):
    """
    Takes a string to a certain scrubber class and returns a python class definition - not an instance.
    """
    try:
        module_name, class_name = path.rsplit(".", 1)
        module = importlib.import_module(module_name)
        return getattr(module, class_name)
    except (ImportError, ValueError) as e:
        raise ImportError(f'Mapped scrubber class "{path}" could not be found.') from e


def _get_model_scrubbers(model):
    # Get model-scrubber-mapping from settings
    scrubber_mapping = settings_with_fallback("SCRUBBER_MAPPING")

    # Initialise scrubber list
    scrubbers = {}

    # Check if model has a settings-defined...
    if model._meta.label in scrubber_mapping:
        scrubber_cls = _parse_scrubber_class_from_string(scrubber_mapping[model._meta.label])
    # If not...
    else:
        # Try to get the scrubber metaclass from the given model
        try:
            scrubber_cls = model.Scrubbers
        except AttributeError:
            return scrubbers  # no model-specific scrubbers

    # Get field mappings from scrubber class
    for k, v in _get_fields(scrubber_cls):
        try:
            field = model._meta.get_field(k)
            scrubbers[field] = v
        except FieldDoesNotExist:
            warnings.warn(f"Scrubber defined for {model.__name__}.{k} but field does not exist", stacklevel=2)

    # Return scrubber-field-mapping
    return scrubbers


def _get_fields(d):
    """
    Helper to get "normal" (i.e.: non-magic and non-dunder) instance attributes.
    Returns an iterator of (field_name, field) tuples.
    """
    return ((k, v) for k, v in vars(d).items() if not k.startswith("_"))


def _filter_out_disabled(d):
    """
    Helper to remove Nones (actually any false-like type) from the scrubbers.
    This is needed so we can disable global scrubbers in a per-model basis.
    """
    return {k: v for k, v in d.items() if v}



import re
from unittest import mock

from django.test import TestCase, override_settings

from django_scrubber import scrubbers
from django_scrubber.services.validator import ScrubberValidatorService


class ScrubberValidatorServiceTest(TestCase):
    class FullUserScrubbers:
        first_name = scrubbers.Hash
        last_name = scrubbers.Hash
        email = scrubbers.Hash
        password = scrubbers.Hash
        username = scrubbers.Hash

    class PartUserScrubbers:
        first_name = scrubbers.Hash
        last_name = scrubbers.Hash
        password = scrubbers.Hash

    def test_process_no_scrubbing(self):
        service = ScrubberValidatorService()
        result = service.process()

        self.assertEqual(len(result), 2)

        model_list = tuple(result.keys())
        self.assertIn("auth.User", model_list)
        self.assertIn("example.DataToBeScrubbed", model_list)

    @override_settings(SCRUBBER_MAPPING={"auth.User": "FullUserScrubbers"})
    @mock.patch(
        "django_scrubber.management.commands.scrub_data._parse_scrubber_class_from_string",
        return_value=FullUserScrubbers,
    )
    def test_process_scrubber_mapper_all_fields(self, mocked_function):
        service = ScrubberValidatorService()
        result = service.process()

        self.assertEqual(len(result), 1)

        model_list = tuple(result.keys())
        self.assertIn("example.DataToBeScrubbed", model_list)

    @override_settings(SCRUBBER_MAPPING={"auth.User": "PartUserScrubbers"})
    @mock.patch(
        "django_scrubber.management.commands.scrub_data._parse_scrubber_class_from_string",
        return_value=PartUserScrubbers,
    )
    def test_process_scrubber_mapper_some_fields(self, mocked_function):
        service = ScrubberValidatorService()
        result = service.process()

        self.assertEqual(len(result), 2)

        model_list = tuple(result.keys())
        self.assertIn("auth.User", model_list)

    @override_settings(SCRUBBER_REQUIRED_FIELD_TYPES=())
    def test_process_scrubber_required_field_type_variable_used(self):
        service = ScrubberValidatorService()
        result = service.process()

        self.assertEqual(len(result), 0)

    @override_settings(
        SCRUBBER_REQUIRED_FIELD_MODEL_WHITELIST=[re.compile("auth.*")],
    )
    def test_process_scrubber_required_field_model_whitelist_regex(self):
        service = ScrubberValidatorService()
        result = service.process()

        model_list = tuple(result.keys())
        self.assertNotIn("auth.User", model_list)
        self.assertNotIn("auth.Permission", model_list)



from io import StringIO
from unittest import mock

from django.core.management import call_command
from django.test import TestCase, override_settings

from django_scrubber.services.validator import ScrubberValidatorService


class TestScrubValidator(TestCase):
    @override_settings(SCRUBBER_STRICT_MODE=False)
    def test_scrub_validator_regular(self):
        out = StringIO()

        with self.assertRaises(SystemExit) as exc:
            call_command(
                "scrub_validation",
                verbosity=3,
                stdout=out,
            )

        self.assertEqual(exc.exception.code, 0)
        self.assertIn("unscrubbed field(s) detected", out.getvalue())
        self.assertIn("However strict mode is deactivated and scrubbing is not enforced.", out.getvalue())

    @override_settings(SCRUBBER_STRICT_MODE=True)
    def test_scrub_validator_strict_mode(self):
        out = StringIO()

        with self.assertRaises(SystemExit) as exc:
            call_command(
                "scrub_validation",
                verbosity=3,
                stdout=out,
            )

        self.assertEqual(exc.exception.code, 1)
        self.assertIn("unscrubbed field(s) detected", out.getvalue())
        self.assertNotIn("However strict mode is deactivated and scrubbing is not enforced.", out.getvalue())

    @mock.patch.object(ScrubberValidatorService, "process")
    def test_scrub_validator_service_called(self, mocked_method):
        out = StringIO()

        call_command("scrub_validation", verbosity=3, stdout=out)

        mocked_method.assert_called_once()
        self.assertIn("No unscrubbed fields detected. Yeah!", out.getvalue())
